// Steam Simulator, A.Greensted
// July 2010
// http://www.labbookpages.co.uk

package steamSim;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import javax.swing.table.*;

public class ParametersPanel extends JPanel implements ListSelectionListener
{
	private ParameterTableModel tableModel;
	private JTable table;
	private SteamEngine engine;
	private Parameter[] paramArray;

	private JTextArea descArea;
	private ListSelectionModel listSelectionModel;

	private boolean altered = false;

	private class ParameterTableModel extends AbstractTableModel
	{
		public int getRowCount()
		{
			if (paramArray == null) return 0;
			return paramArray.length;
		}

		public int getColumnCount()
		{
			return 2;
		}

		public Class getColumnClass(int column)
		{
			switch (column)
			{
				case 0: return String.class;
				case 1: return Double.class;
			}

			return null;
		}

		public String getColumnName(int column)
		{
			switch (column)
			{
				case 0: return "Parameter";
				case 1: return "Value(mm)";
			}

			return null;
		}

		public Object getValueAt(int row, int column)
		{
			if (paramArray == null || row < 0 || row >= paramArray.length) return null;

			switch (column)
			{
				case 0:	return paramArray[row].name;
				case 1:	return paramArray[row].value;
			}

			return null;
		}

		public void setValueAt(Object value, int row, int column)
		{
			if (column != 1) return;
			if (paramArray == null || row < 0 || row >= paramArray.length) return;

			try
			{
				paramArray[row].value = (Double)value;
				altered = true;
				engine.repaint();
			}
			catch (Exception e)
			{
				System.err.println(e);
			}
		}

		public boolean isCellEditable(int row, int column)
		{
			if (column == 1) return true;
			return false;
		}
	}

	public ParametersPanel()
	{
		super(new BorderLayout(5,5));

		tableModel = new ParameterTableModel();
		table = new JTable(tableModel);

		TableColumnModel model = table.getColumnModel();
		model.getColumn(0).setPreferredWidth(180);
		model.getColumn(1).setPreferredWidth(80);

		table.setCellSelectionEnabled(true);

		table.setDefaultRenderer(Double.class, new DefaultTableCellRenderer() {
			public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
			{
				Component c = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
				if (paramArray != null && row>=0 && row<paramArray.length) {
					if (paramArray[row].colour != null) c.setBackground(paramArray[row].colour);
				}
				return c;
			}
		});

		table.setDefaultRenderer(String.class, new DefaultTableCellRenderer() {
			public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
			{
				Component c = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
				if (paramArray != null && row>=0 && row<paramArray.length) {
					if (paramArray[row].colour != null) c.setBackground(paramArray[row].colour);
				}
				return c;
			}
		});

		listSelectionModel = table.getSelectionModel();
		listSelectionModel.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		listSelectionModel.addListSelectionListener(this);

		JScrollPane scrollPane = new JScrollPane(table);
		scrollPane.setPreferredSize(new Dimension(300, 400));
		table.setFillsViewportHeight(true);

		descArea = new JTextArea(4, 20);
		descArea.setLineWrap(true);
		descArea.setWrapStyleWord(true);
		descArea.setEditable(false);
		descArea.setText("Select a parameter");
		descArea.setBackground(new Color(238,238,238));
		descArea.setBorder(new TitledBorder("Parameter Description"));

		add(scrollPane, BorderLayout.CENTER);
		add(descArea, BorderLayout.SOUTH);
	}

	public boolean getAltered()
	{
		return altered;
	}

	public void setToUnaltered()
	{
		altered = false;
	}

	public void setEngine(SteamEngine engine)
	{
		this.engine = engine;
		altered = false;
		paramArray = engine.getParametersArray();
		tableModel.fireTableDataChanged(); 
	}

	public void valueChanged(ListSelectionEvent e)
	{
		if (paramArray == null) return;

		int row = listSelectionModel.getMinSelectionIndex();
		if (row<0 || row>=paramArray.length) return;

		descArea.setText(paramArray[row].description);
	}
}
