// Steam Simulator, A.Greensted
// July 2010
// http://www.labbookpages.co.uk

package steamSim;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.print.*;

public class SteamSimulator implements ActionListener
{
	private static boolean WEB_START = false;

	static
	{
		// Determine if running from Web Start
		try
		{
			Class test = Class.forName("javax.jnlp.ServiceManager");
			WEB_START = true;
		}
		catch (Exception e) {}

		System.out.println("Running Webstart: " + WEB_START);
	}

	private EngineSelector selector;
	private SteamEngine engine;

	private JFrame frame;
	private JButton newButton;
	private JButton openButton;
	private JButton saveButton;
	private JButton printButton;

	private ParametersPanel paramPanel;

	private JPanel animContainer;
	private JLabel blankLabel;

	private JButton playButton;
	private JButton stepBackButton;
	private JButton stepForwardButton;
	private JButton centreViewButton;
	private JButton resetAnimButton;
	private Icon playIcon;
	private Icon stopIcon;

	private Timer timer;

	private boolean running = false;

	public SteamSimulator(String filename)
	{
		timer = new Timer(50, this);

		frame = new JFrame("Steam Engine Simulator");
		frame.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		frame.addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{
				if (paramPanel.getAltered())
				{
					Object[] options = new Object[] {"Allow me to save", "Just exit"};

					int option = JOptionPane.showOptionDialog(frame,
						"Engine parameters have been altered.\nDo you want to exit without saving?",
						"Exit without saving?",
						JOptionPane.YES_NO_CANCEL_OPTION,
						JOptionPane.QUESTION_MESSAGE,
						null,
						options,
						options[0]);

					if (option == 0) return;
				}

				System.exit(0);
			}
		});

		setIcons(frame);
	
		selector = new EngineSelector(frame);

		paramPanel = new ParametersPanel();

		// File Panel
		JPanel filePanel = new JPanel();

		newButton = createButton("New", "/icons/new.png", true, true);
		newButton.addActionListener(this);
		filePanel.add(newButton);

		openButton = createButton("Open", "/icons/open.png", true, true);
		openButton.addActionListener(this);
		filePanel.add(openButton);

		saveButton = createButton("Save", "/icons/save.png", false, true);
		saveButton.addActionListener(this);
		filePanel.add(saveButton);

		printButton = createButton("Print", "/icons/print.png", false, true);
		printButton.addActionListener(this);
		filePanel.add(printButton);

		// Control Panel
		JPanel controlPanel = new JPanel();

		java.net.URL imgURL = getClass().getResource("/icons/play.png");
		if (imgURL != null) playIcon = new ImageIcon(imgURL);
		imgURL = getClass().getResource("/icons/stop.png");
		if (imgURL != null) stopIcon = new ImageIcon(imgURL);

		stepBackButton = createButton("Step Back", "/icons/stepBackward.png", false, false);
		stepBackButton.addActionListener(this);
		controlPanel.add(stepBackButton);

		playButton = createButton("Play", null, false, false);
		playButton.setIcon(playIcon);
		playButton.addActionListener(this);
		controlPanel.add(playButton);

		stepForwardButton = createButton("Step Forward", "/icons/stepForward.png", false, false);
		stepForwardButton.addActionListener(this);
		controlPanel.add(stepForwardButton);

		resetAnimButton = createButton("Reset Animation", "/icons/reset.png", false, false);
		resetAnimButton.addActionListener(this);
		controlPanel.add(resetAnimButton);

		centreViewButton = createButton("Centre View", "/icons/centreView.png", false, false);
		centreViewButton.addActionListener(this);
		controlPanel.add(centreViewButton);

		// Info Panel
		Box infoBox = new Box(BoxLayout.X_AXIS);
		infoBox.setBorder(new CompoundBorder(new LineBorder(Color.LIGHT_GRAY), new EmptyBorder(3,5,3,5)));
		JLabel label = new JLabel("A.Greensted  ", SwingConstants.LEFT);
		infoBox.add(label);
		Font font = label.getFont().deriveFont(Font.PLAIN);
		label = new JLabel("http://www.labbookpages.co.uk", SwingConstants.LEFT);
		label.setFont(font);
		infoBox.add(label);
		infoBox.add(Box.createHorizontalGlue());
		label = new JLabel("Version 0.1", SwingConstants.RIGHT);
		infoBox.add(label);

		// Put everything together
		animContainer = new JPanel(new BorderLayout());
		blankLabel = new JLabel("Click 'New' and select engine", JLabel.CENTER);
		blankLabel.setPreferredSize(SteamEngine.PREFERRED_DIMENSION);
		animContainer.add(blankLabel, BorderLayout.CENTER);
		animContainer.add(controlPanel, BorderLayout.SOUTH);

		JPanel rightPanel = new JPanel(new BorderLayout());
		rightPanel.add(filePanel, BorderLayout.NORTH);
		rightPanel.add(paramPanel, BorderLayout.CENTER);

		JPanel mainContainer = new JPanel(new BorderLayout(5,5));
		mainContainer.setBorder(new EmptyBorder(5,5,5,5));

		mainContainer.add(animContainer, BorderLayout.CENTER);
		mainContainer.add(rightPanel, BorderLayout.EAST);
		mainContainer.add(infoBox, BorderLayout.SOUTH);

		frame.getContentPane().add(mainContainer);
		frame.pack();

		// Load file if specified
		if (filename != null)
		{
			try
			{
				FileInputStream stream = new FileInputStream(filename);
				if (stream == null) return;

				SteamEngine newEngine = SteamEngine.readFromStream(stream);
				if (newEngine == null) return;

				replaceEngine(newEngine);
			}
			catch (Exception e)
			{
				System.err.println(e);
				JOptionPane.showMessageDialog(frame, "Could not open file: " + filename, "File Error", JOptionPane.ERROR_MESSAGE);
			}
		}

		// Display GUI
		frame.setVisible(true);
	}

	private void setIcons(JFrame frame)
	{
		String[] suffixes = new String[] {"16", "22", "32", "64", "128", "256"};
		java.util.ArrayList<Image> imageList = new java.util.ArrayList<Image>();

		for (String suffix : suffixes) {
			java.net.URL imgURL = getClass().getResource("/icons/icon_" + suffix + ".png");
			if (imgURL != null) imageList.add((new ImageIcon(imgURL)).getImage());
		}

		frame.setIconImages(imageList);
	}

	private JButton createButton(String label, String iconName, boolean enabled, boolean below)
	{
		JButton button = new JButton(label);

		button.setEnabled(enabled);

		if (iconName != null) {
			java.net.URL imgURL = getClass().getResource(iconName);
			if (imgURL != null) button.setIcon(new ImageIcon(imgURL));
		}

		if (below) {
			button.setVerticalTextPosition(SwingConstants.BOTTOM);
			button.setHorizontalTextPosition(SwingConstants.CENTER);
		}

		return button;
	}

	public void actionPerformed(ActionEvent aE)
	{
		Object o = aE.getSource();
		
		if (o == timer)
		{
			engine.stepAnimationForward();
			engine.repaint();
			return;
		}

		if (o == playButton)
		{
			if (timer.isRunning())
			{
				timer.stop();
				playButton.setIcon(playIcon);
				playButton.setText("Play");
				stepBackButton.setEnabled(true);
				stepForwardButton.setEnabled(true);
			}
			else
			{
				stepBackButton.setEnabled(false);
				stepForwardButton.setEnabled(false);
				playButton.setIcon(stopIcon);
				playButton.setText("Stop");
				timer.start();
			}
			return;
		}

		if (o == stepBackButton)
		{
			engine.stepAnimationBack();
			engine.repaint();
			return;
		}

		if (o == stepForwardButton)
		{
			engine.stepAnimationForward();
			engine.repaint();
			return;
		}

		if (o == resetAnimButton)
		{
			engine.resetAnimation();
			engine.repaint();
			return;
		}

		if (o == centreViewButton)
		{
			engine.centreView();
			engine.repaint();
			return;
		}

		if (o == newButton)
		{
			if (paramPanel.getAltered())
			{
				Object[] options = new Object[] {"Allow me to save", "Continue"};

				int option = JOptionPane.showOptionDialog(frame,
					"Engine parameters have been altered.\nDo you want to continue without saving?",
					"Continue?",
					JOptionPane.YES_NO_CANCEL_OPTION,
					JOptionPane.QUESTION_MESSAGE,
					null,
					options,
					options[0]);

				if (option == 0) return;
			}

			Class engineClass = selector.showSelector();
			if (engineClass == null) return;

			SteamEngine newEngine = null;

			try
			{
				newEngine = (SteamEngine) engineClass.newInstance();
			}
			catch (Exception e)
			{
				System.err.println(e);
			}

			if (newEngine == null) return;
			replaceEngine(newEngine);

			return;
		}

		if (o == openButton)
		{
			if (paramPanel.getAltered())
			{
				Object[] options = new Object[] {"Allow me to save", "Continue"};

				int option = JOptionPane.showOptionDialog(frame,
					"Engine parameters have been altered.\nDo you want to continue without saving?",
					"Continue?",
					JOptionPane.YES_NO_CANCEL_OPTION,
					JOptionPane.QUESTION_MESSAGE,
					null,
					options,
					options[0]);

				if (option == 0) return;
			}

			
			InputStream stream = (WEB_START) ? WSStreamHandler.getOpenStream(frame) : StreamHandler.getOpenStream(frame);
			if (stream == null) return;

			SteamEngine newEngine = SteamEngine.readFromStream(stream);
			if (newEngine == null) return;

			replaceEngine(newEngine);
			return;
		}

		if (o == saveButton)
		{
			OutputStream stream = (WEB_START) ? WSStreamHandler.getSaveStream(frame) : StreamHandler.getSaveStream(frame);
			if (stream == null) return;

			SteamEngine.writeToStream(stream, engine);
			paramPanel.setToUnaltered();
			return;
		}

		if (o == printButton)
		{
			PrinterJob job = PrinterJob.getPrinterJob();
			job.setPrintable(engine);

			if (job.printDialog())
			{
				try
				{
					job.print();
				}
				catch (PrinterException exc)
				{
					System.out.println(exc);
				}
			}

			return;
		}
	}

	private void replaceEngine(SteamEngine newEngine)
	{
		if (engine == null) animContainer.remove(blankLabel);
		else animContainer.remove(engine);

		engine = newEngine;
	
		animContainer.add(engine, BorderLayout.CENTER);
		paramPanel.setEngine(engine);

		playButton.setEnabled(true);
		stepBackButton.setEnabled(true);
		stepForwardButton.setEnabled(true);
		centreViewButton.setEnabled(true);
		resetAnimButton.setEnabled(true);
		saveButton.setEnabled(true);
		printButton.setEnabled(true);

		animContainer.revalidate();
		engine.repaint();
	}

	public static void main(String[] args)
	{
		String filename = null;
		if (args.length >= 1) filename = args[0];
		new SteamSimulator(filename);
	}
}
